@@ -12,11 +12,11 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+from io import StringIO
 import threading
+from unittest import mock
 
-import mock
 from oslo_config import cfg
-import six
 
 from ceilometermiddleware import swift
 from ceilometermiddleware.tests import base as tests_base
@@ -53,9 +53,9 @@ class FakeRequest(object):
         environ['PATH_INFO'] = path
 
         if 'wsgi.input' not in environ:
-            environ['wsgi.input'] = six.moves.cStringIO('')
+            environ['wsgi.input'] = StringIO('')
 
-        for header, value in six.iteritems(headers):
+        for header, value in headers.items():
             environ['HTTP_%s' % header.upper()] = value
         self.environ = environ
 
@@ -70,12 +70,15 @@ class TestSwift(tests_base.TestCase):
 
     @staticmethod
     def start_response(*args):
-            pass
+        pass
+
+    def get_request(self, path, environ=None, headers=None):
+        return FakeRequest(path, environ=environ, headers=headers)
 
     def test_get(self):
         app = swift.Swift(FakeApp(), {})
-        req = FakeRequest('/1.0/account/container/obj',
-                          environ={'REQUEST_METHOD': 'GET'})
+        req = self.get_request('/1.0/account/container/obj',
+                               environ={'REQUEST_METHOD': 'GET'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             resp = app(req.environ, self.start_response)
             self.assertEqual(["This string is 28 bytes long"], list(resp))
@@ -96,8 +99,8 @@ class TestSwift(tests_base.TestCase):
         app = swift.Swift(FakeApp(),
                           {"nonblocking_notify": "True",
                            "send_queue_size": "1"})
-        req = FakeRequest('/1.0/account/container/obj',
-                          environ={'REQUEST_METHOD': 'GET'})
+        req = self.get_request('/1.0/account/container/obj',
+                               environ={'REQUEST_METHOD': 'GET'})
         with mock.patch('oslo_messaging.Notifier.info',
                         side_effect=lambda *args, **kwargs: notified.set()
                         ) as notify:
@@ -118,11 +121,11 @@ class TestSwift(tests_base.TestCase):
 
     def test_put(self):
         app = swift.Swift(FakeApp(body=['']), {})
-        req = FakeRequest(
+        req = self.get_request(
             '/1.0/account/container/obj',
             environ={'REQUEST_METHOD': 'PUT',
                      'wsgi.input':
-                     six.moves.cStringIO('some stuff')})
+                     StringIO('some stuff')})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -139,10 +142,10 @@ class TestSwift(tests_base.TestCase):
 
     def test_post(self):
         app = swift.Swift(FakeApp(body=['']), {})
-        req = FakeRequest(
+        req = self.get_request(
             '/1.0/account/container/obj',
             environ={'REQUEST_METHOD': 'POST',
-                     'wsgi.input': six.moves.cStringIO('some other stuff')})
+                     'wsgi.input': StringIO('some other stuff')})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -159,8 +162,8 @@ class TestSwift(tests_base.TestCase):
 
     def test_head(self):
         app = swift.Swift(FakeApp(body=['']), {})
-        req = FakeRequest('/1.0/account/container/obj',
-                          environ={'REQUEST_METHOD': 'HEAD'})
+        req = self.get_request('/1.0/account/container/obj',
+                               environ={'REQUEST_METHOD': 'HEAD'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -176,8 +179,8 @@ class TestSwift(tests_base.TestCase):
     def test_bogus_request(self):
         """Test even for arbitrary request method, this will still work."""
         app = swift.Swift(FakeApp(body=['']), {})
-        req = FakeRequest('/1.0/account/container/obj',
-                          environ={'REQUEST_METHOD': 'BOGUS'})
+        req = self.get_request('/1.0/account/container/obj',
+                               environ={'REQUEST_METHOD': 'BOGUS'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -192,8 +195,8 @@ class TestSwift(tests_base.TestCase):
 
     def test_get_container(self):
         app = swift.Swift(FakeApp(), {})
-        req = FakeRequest('/1.0/account/container',
-                          environ={'REQUEST_METHOD': 'GET'})
+        req = self.get_request('/1.0/account/container',
+                               environ={'REQUEST_METHOD': 'GET'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -210,8 +213,8 @@ class TestSwift(tests_base.TestCase):
 
     def test_no_metadata_headers(self):
         app = swift.Swift(FakeApp(), {})
-        req = FakeRequest('/1.0/account/container',
-                          environ={'REQUEST_METHOD': 'GET'})
+        req = self.get_request('/1.0/account/container',
+                               environ={'REQUEST_METHOD': 'GET'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -230,11 +233,11 @@ class TestSwift(tests_base.TestCase):
         app = swift.Swift(FakeApp(), {
             'metadata_headers': 'X_VAR1, x-var2, x-var3, token'
         })
-        req = FakeRequest('/1.0/account/container',
-                          environ={'REQUEST_METHOD': 'GET'},
-                          headers={'X_VAR1': 'value1',
-                                   'X_VAR2': 'value2',
-                                   'TOKEN': 'token'})
+        req = self.get_request('/1.0/account/container',
+                               environ={'REQUEST_METHOD': 'GET'},
+                               headers={'X_VAR1': 'value1',
+                                        'X_VAR2': 'value2',
+                                        'TOKEN': 'token'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -258,9 +261,9 @@ class TestSwift(tests_base.TestCase):
             'metadata_headers': 'unicode'
         })
         uni = u'\xef\xbd\xa1\xef\xbd\xa5'
-        req = FakeRequest('/1.0/account/container',
-                          environ={'REQUEST_METHOD': 'GET'},
-                          headers={'UNICODE': uni})
+        req = self.get_request('/1.0/account/container',
+                               environ={'REQUEST_METHOD': 'GET'},
+                               headers={'UNICODE': uni})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -274,15 +277,15 @@ class TestSwift(tests_base.TestCase):
             http_headers = [k for k in metadata.keys()
                             if k.startswith('http_header_')]
             self.assertEqual(1, len(http_headers))
-            self.assertEqual(six.text_type(uni),
+            self.assertEqual(str(uni),
                              metadata['http_header_unicode'])
 
     def test_metadata_headers_on_not_existing_header(self):
         app = swift.Swift(FakeApp(), {
             'metadata_headers': 'x-var3'
         })
-        req = FakeRequest('/1.0/account/container',
-                          environ={'REQUEST_METHOD': 'GET'})
+        req = self.get_request('/1.0/account/container',
+                               environ={'REQUEST_METHOD': 'GET'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -312,12 +315,12 @@ class TestSwift(tests_base.TestCase):
             list(app(req.environ, self.start_response))
             self.assertEqual(0, len(notify.call_args_list))
 
-    @mock.patch('six.moves.urllib.parse.quote')
+    @mock.patch('urllib.parse.quote')
     def test_emit_event_fail(self, mocked_func):
         mocked_func.side_effect = Exception("a exception")
         app = swift.Swift(FakeApp(body=["test"]), {})
-        req = FakeRequest('/1.0/account/container',
-                          environ={'REQUEST_METHOD': 'GET'})
+        req = self.get_request('/1.0/account/container',
+                               environ={'REQUEST_METHOD': 'GET'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             resp = list(app(req.environ, self.start_response))
             self.assertEqual(0, len(notify.call_args_list))
@@ -325,8 +328,8 @@ class TestSwift(tests_base.TestCase):
 
     def test_reseller_prefix(self):
         app = swift.Swift(FakeApp(), {})
-        req = FakeRequest('/1.0/AUTH_account/container/obj',
-                          environ={'REQUEST_METHOD': 'GET'})
+        req = self.get_request('/1.0/AUTH_account/container/obj',
+                               environ={'REQUEST_METHOD': 'GET'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertEqual(1, len(notify.call_args_list))
@@ -426,7 +429,7 @@ class TestSwift(tests_base.TestCase):
             '/1.0/account/container/obj',
             environ={'REQUEST_METHOD': 'PUT',
                      'wsgi.input':
-                     six.moves.cStringIO('some stuff'),
+                     StringIO('some stuff'),
                      'swift.source': 'RL'})
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
@@ -459,3 +462,14 @@ class TestSwift(tests_base.TestCase):
                          app.ignore_projects)
         warning.assert_called_once_with(
             "fail to find project '%s' in keystone", "gnocchi")
+
+
+class TestSwiftS3Api(TestSwift):
+
+    def get_request(self, path, environ=None, headers=None):
+        # Add Swift Path in environ, provided by swift s3api middleware
+        environ['swift.backend_path'] = path
+        # Emulate S3 api PATH_INFO by removing /v1 and account parts
+        path = '/' + path.split('/', 3)[-1]
+
+        return FakeRequest(path, environ=environ, headers=headers)
